"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _types = require("./types");

var _fs = _interopRequireDefault(require("fs"));

var _util = require("@polkadot/util");

var _defaults = _interopRequireDefault(require("./defaults"));

// Copyright 2017-2019 @polkadot/client-db authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
const l = (0, _util.logger)('db/compact');

class Compact {
  constructor(file) {
    this._fd = void 0;
    this._file = void 0;
    this._fd = -1;
    this._file = file;
  }

  maintain(fn) {
    (0, _util.assert)(this._fd === -1, 'Database cannot be open for compacting');
    l.log('compacting database');
    const start = Date.now();
    const newFile = "".concat(this._file, ".compacted");

    const newFd = this._open(newFile, true);

    const oldFd = this._open(this._file);

    const keys = this._compact(fn, newFd, oldFd);

    _fs.default.closeSync(oldFd);

    _fs.default.closeSync(newFd);

    const newStat = _fs.default.lstatSync(newFile);

    const oldStat = _fs.default.lstatSync(this._file);

    const percentage = 100 * (newStat.size / oldStat.size);
    const sizeMB = newStat.size / (1024 * 1024);
    const elapsed = (Date.now() - start) / 1000;

    _fs.default.unlinkSync(this._file);

    _fs.default.renameSync(newFile, this._file);

    l.log("compacted in ".concat(elapsed.toFixed(2), "s, ").concat((keys / 1000).toFixed(2), "k keys, ").concat(sizeMB.toFixed(2), "MB (").concat(percentage.toFixed(2), "%)"));
  }

  _open(file) {
    let startEmpty = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

    if (!_fs.default.existsSync(file) || startEmpty) {
      _fs.default.writeFileSync(file, Buffer.alloc(_defaults.default.BRANCH_SIZE));
    }

    return _fs.default.openSync(file, 'r+');
  }

  _compact(fn, newFd, oldFd) {
    // l.debug(() => ['_compact', debug({ newFd, oldFd, newAt, oldAt })]);
    let keys = 0;
    let percent = 0;

    const doCompact = (newAt, oldAt, depth) => {
      const increment = 100 / _defaults.default.ENTRY_NUM / Math.pow(_defaults.default.ENTRY_NUM, depth);

      for (let index = 0; index < _defaults.default.ENTRY_NUM; index++) {
        const entry = this._compactReadEntry(oldFd, oldAt, index);

        const dataAt = entry.readUIntBE(1, _defaults.default.UINT_SIZE);
        const entryType = entry[0];

        if (entryType === _types.Slot.EMPTY) {
          // l.debug(() => '_compact/isEmpty');
          percent += increment;
        } else if (entryType === _types.Slot.LEAF) {
          // l.debug(() => '_compact/isLeaf');
          const _this$_compactReadKey = this._compactReadKey(oldFd, dataAt),
                _this$_compactReadKey2 = (0, _slicedToArray2.default)(_this$_compactReadKey, 2),
                key = _this$_compactReadKey2[0],
                value = _this$_compactReadKey2[1];

          const keyAt = this._compactWriteKey(newFd, key, value);

          this._compactUpdateLink(newFd, newAt, index, keyAt, _types.Slot.LEAF);

          keys++;
          percent += increment;
        } else if (entryType === _types.Slot.BRANCH) {
          // l.debug(() => '_compact/isBranch');
          const headerAt = this._compactWriteHeader(newFd, newAt, index);

          doCompact(headerAt, dataAt, depth + 1);
        } else {
          throw new Error("Unknown entry type, ".concat(entryType));
        }

        fn({
          isCompleted: depth === 0 && index === _defaults.default.ENTRY_NUM - 1,
          keys,
          percent
        });
      } // l.debug(() => ['_compact', '=>', `${depth}: ${keys} keys written`]);

    };

    doCompact(0, 0, 0);
    return keys;
  }

  _compactReadEntry(fd, at, index) {
    const entry = Buffer.alloc(_defaults.default.ENTRY_SIZE);
    const entryAt = at + index * _defaults.default.ENTRY_SIZE;

    _fs.default.readSync(fd, entry, 0, _defaults.default.ENTRY_SIZE, entryAt);

    return entry;
  }

  _compactReadKey(fd, at) {
    const key = Buffer.alloc(_defaults.default.KEY_TOTAL_SIZE);

    _fs.default.readSync(fd, key, 0, _defaults.default.KEY_TOTAL_SIZE, at);

    const valueLength = key.readUIntBE(_defaults.default.KEY_SIZE, _defaults.default.UINT_SIZE);
    const valueAt = key.readUIntBE(_defaults.default.KEY_SIZE + _defaults.default.UINT_SIZE, _defaults.default.UINT_SIZE);
    const value = Buffer.alloc(valueLength);

    _fs.default.readSync(fd, value, 0, valueLength, valueAt);

    return [key, value];
  }

  _compactWriteKey(fd, key, value) {
    const valueAt = _fs.default.fstatSync(fd).size;

    const keyAt = valueAt + value.length;
    key.writeUIntBE(valueAt, _defaults.default.KEY_SIZE + _defaults.default.UINT_SIZE, _defaults.default.UINT_SIZE);

    _fs.default.writeSync(fd, value, 0, value.length, valueAt);

    _fs.default.writeSync(fd, key, 0, _defaults.default.KEY_TOTAL_SIZE, keyAt);

    return keyAt;
  }

  _compactUpdateLink(fd, at, index, pointer, type) {
    const entry = Buffer.alloc(_defaults.default.ENTRY_SIZE);
    entry.set([type], 0);
    entry.writeUIntBE(pointer, 1, _defaults.default.UINT_SIZE);

    _fs.default.writeSync(fd, entry, 0, _defaults.default.ENTRY_SIZE, at + index * _defaults.default.ENTRY_SIZE);
  }

  _compactWriteHeader(fd, at, index) {
    const headerAt = _fs.default.fstatSync(fd).size;

    const header = Buffer.alloc(_defaults.default.BRANCH_SIZE);

    _fs.default.writeSync(fd, header, 0, _defaults.default.BRANCH_SIZE, headerAt);

    this._compactUpdateLink(fd, at, index, headerAt, _types.Slot.BRANCH);

    return headerAt;
  }

}

exports.default = Compact;