"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _types = require("./types");

var _fs = _interopRequireDefault(require("fs"));

var _util = require("@polkadot/util");

var _Cache = _interopRequireDefault(require("./Cache"));

var _defaults = _interopRequireDefault(require("./defaults"));

// Copyright 2017-2019 @polkadot/client-db authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
const l = (0, _util.logger)('db/flat');

class Impl extends _Cache.default {
  _getKeyValue(keyAt) {
    return this._getCachedData(keyAt, _defaults.default.KEY_TOTAL_SIZE);
  }

  _retrieveBranch(doCreate, branch, entryIndex, keyIndex, key) {
    const nextBranchAt = branch.readUIntBE(entryIndex + 1, _defaults.default.UINT_SIZE);
    l.debug(() => ['findKey/isBranch', {
      nextBranchAt
    }]);
    return this._findKey(key, doCreate, keyIndex + 1, nextBranchAt);
  }

  _retrieveEmpty(doCreate, branch, branchAt, entryIndex, key) {
    l.debug(() => ['findKey/isEmpty']);
    return doCreate ? this.writeNewLeaf(branch, branchAt, entryIndex, key) : null;
  }

  _retrieveLeaf(doCreate, branch, branchAt, entryIndex, keyIndex, key) {
    const keyAt = branch.readUIntBE(entryIndex + 1, _defaults.default.UINT_SIZE);

    const keyValue = this._getKeyValue(keyAt);

    const prevKey = this._serializeKey(keyValue.subarray(0, _defaults.default.KEY_SIZE));

    let matchIndex = keyIndex;
    l.debug(() => ['findKey/isLeaf', {
      keyAt,
      branch,
      branchAt,
      entryIndex,
      keyValue
    }]);

    while (matchIndex < _defaults.default.KEY_PARTS_LENGTH) {
      if (prevKey.parts[matchIndex] !== key.parts[matchIndex]) {
        break;
      }

      matchIndex++;
    }

    if (matchIndex !== _defaults.default.KEY_PARTS_LENGTH) {
      return doCreate ? this.writeNewBranch(branch, branchAt, entryIndex, key, keyAt, prevKey, matchIndex, matchIndex - keyIndex - 1) : null;
    }

    return {
      key,
      keyAt,
      keyValue
    };
  }

  _findKey(key, doCreate, keyIndex, branchAt) {
    const entryIndex = key.parts[keyIndex] * _defaults.default.ENTRY_SIZE;

    const branch = this._getCachedBranch(branchAt);

    l.debug(() => ['findKey', {
      key,
      doCreate,
      keyIndex,
      branchAt,
      branch,
      entryIndex
    }]);
    const entryType = branch[entryIndex];

    switch (entryType) {
      case _types.Slot.BRANCH:
        return this._retrieveBranch(doCreate, branch, entryIndex, keyIndex, key);

      case _types.Slot.EMPTY:
        return this._retrieveEmpty(doCreate, branch, branchAt, entryIndex, key);

      case _types.Slot.LEAF:
        return this._retrieveLeaf(doCreate, branch, branchAt, entryIndex, keyIndex, key);

      default:
        throw new Error("Unhandled entry type ".concat(entryType));
    }
  }

  _extractValueInfo(keyValue) {
    return {
      valueLength: keyValue.readUIntBE(_defaults.default.KEY_SIZE, _defaults.default.UINT_SIZE),
      valueAt: keyValue.readUIntBE(_defaults.default.KEY_SIZE + _defaults.default.UINT_SIZE, _defaults.default.UINT_SIZE)
    };
  }

  _readValue(keyValue) {
    l.debug(() => ['readValue', {
      keyValue
    }]);

    const _this$_extractValueIn = this._extractValueInfo(keyValue),
          valueAt = _this$_extractValueIn.valueAt,
          valueLength = _this$_extractValueIn.valueLength;

    const value = this._getCachedData(valueAt, valueLength);

    return {
      value,
      valueAt
    };
  }

  _writeValue(keyAt, keyValue, value) {
    // l.debug(() => ['writeValue', { keyAt, keyValue, value }]);
    const current = this._extractValueInfo(keyValue);

    const valueAt = value.length > current.valueLength ? this._writeNewBuffer(value) : this._writeUpdatedBuffer(value, current.valueAt);
    keyValue.writeUIntBE(value.length, _defaults.default.KEY_SIZE, _defaults.default.UINT_SIZE);
    keyValue.writeUIntBE(valueAt, _defaults.default.KEY_SIZE + _defaults.default.UINT_SIZE, _defaults.default.UINT_SIZE);

    _fs.default.writeSync(this._fd, keyValue, _defaults.default.KEY_SIZE, 2 * _defaults.default.UINT_SIZE, keyAt + _defaults.default.KEY_SIZE);

    return {
      value,
      valueAt
    };
  }

  _writeNewKey(key) {
    // l.debug(() => ['writeNewKey', { key }]);
    const keyValue = Buffer.alloc(_defaults.default.KEY_TOTAL_SIZE);
    keyValue.set(key.buffer, 0);

    const keyAt = this._writeNewBuffer(keyValue);

    return {
      key,
      keyAt,
      keyValue
    };
  }

  writeNewKey(key) {
    const result = this._writeNewKey(key);

    l.debug(() => ['writeNewKey', '=>', {
      result
    }]);
    return result;
  }

  _writeNewBranch(branch, branchAt, entryIndex, key, prevAt, prevKey, matchIndex, depth) {
    // l.debug(() => ['writeNewBranch', { branch, branchAt, entryIndex, key, prevAt, prevKey, matchIndex, depth }]);
    const _this$writeNewKey = this.writeNewKey(key),
          keyAt = _this$writeNewKey.keyAt,
          keyValue = _this$writeNewKey.keyValue;

    const keyIndex = key.parts[matchIndex] * _defaults.default.ENTRY_SIZE;
    const prevIndex = prevKey.parts[matchIndex] * _defaults.default.ENTRY_SIZE;
    const buffers = [];
    let newBranchAt = this._fileSize;
    let newBranch = Buffer.alloc(_defaults.default.BRANCH_SIZE);
    newBranch.set([_types.Slot.LEAF], keyIndex);
    newBranch.writeUIntBE(keyAt, keyIndex + 1, _defaults.default.UINT_SIZE);
    newBranch.set([_types.Slot.LEAF], prevIndex);
    newBranch.writeUIntBE(prevAt, prevIndex + 1, _defaults.default.UINT_SIZE);
    buffers.push(newBranch);

    for (let offset = 1; depth > 0; depth--, offset++) {
      const branchIndex = key.parts[matchIndex - offset] * _defaults.default.ENTRY_SIZE;
      newBranch = Buffer.alloc(_defaults.default.BRANCH_SIZE);
      newBranch.set([_types.Slot.BRANCH], branchIndex);
      newBranch.writeUIntBE(newBranchAt, branchIndex + 1, _defaults.default.UINT_SIZE);
      buffers.push(newBranch);
      newBranchAt += _defaults.default.BRANCH_SIZE;
    }

    this._writeNewBuffers(buffers);

    branch.set([_types.Slot.BRANCH], entryIndex);
    branch.writeUIntBE(newBranchAt, entryIndex + 1, _defaults.default.UINT_SIZE);

    _fs.default.writeSync(this._fd, branch, entryIndex, _defaults.default.ENTRY_SIZE, branchAt + entryIndex);

    return {
      key,
      keyAt,
      keyValue
    };
  }

  writeNewBranch(branch, branchAt, entryIndex, key, prevAt, prevValue, matchIndex, depth) {
    const result = this._writeNewBranch(branch, branchAt, entryIndex, key, prevAt, prevValue, matchIndex, depth);

    l.debug(() => ['writeNewBranch', '=>', {
      result
    }]);
    return result;
  }

  _writeNewLeaf(branch, branchAt, entryIndex, key) {
    l.debug(() => ['writeNewLeaf', {
      branch,
      branchAt,
      entryIndex,
      key
    }]);

    const _this$writeNewKey2 = this.writeNewKey(key),
          keyAt = _this$writeNewKey2.keyAt,
          keyValue = _this$writeNewKey2.keyValue;

    branch.set([_types.Slot.LEAF], entryIndex);
    branch.writeUIntBE(keyAt, entryIndex + 1, _defaults.default.UINT_SIZE);

    _fs.default.writeSync(this._fd, branch, entryIndex, _defaults.default.ENTRY_SIZE, branchAt + entryIndex);

    return {
      key,
      keyAt,
      keyValue
    };
  }

  writeNewLeaf(branch, branchAt, entryIndex, key) {
    const result = this._writeNewLeaf(branch, branchAt, entryIndex, key);

    l.debug(() => ['writeNewLeaf', '=>', {
      result
    }]);
    return result;
  }

  _writeUpdatedBuffer(buffer, bufferAt) {
    _fs.default.writeSync(this._fd, buffer, 0, buffer.length, bufferAt);

    this._cacheData(bufferAt, buffer);

    return bufferAt;
  }

  _writeNewBuffer(buffer) {
    let withCache = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
    const startAt = this._fileSize;

    _fs.default.writeSync(this._fd, buffer, 0, buffer.length, startAt);

    if (withCache) {
      this._cacheData(startAt, buffer);
    }

    this._fileSize += buffer.length;
    return startAt;
  }

  _writeNewBuffers(buffers) {
    let bufferAt = this._fileSize;
    buffers.forEach(buffer => {
      this._cacheData(bufferAt, buffer);

      bufferAt += buffer.length;
    });
    return this._writeNewBuffer(Buffer.concat(buffers), false);
  }

}

exports.default = Impl;