"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _fs = _interopRequireDefault(require("fs"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _nodeLmdb = _interopRequireDefault(require("node-lmdb"));

var _path = _interopRequireDefault(require("path"));

var _util = require("@polkadot/util");

// Copyright 2017-2019 @polkadot/client-db authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
const GB = 1 * 1024 * 1024 * 1024;
const l = (0, _util.logger)('db/lmdb'); // NOTE we manage 2 transaction types, one for read and one for write. For
// triedbs we assume that the transaction _only_ contains writes (managed by
// the transaction overlay). At the same time have a read that we keep open
// so we don't need to re-create the transaction, otherwise wastefull.
//
// There is reasoning behind this madness, reads cannot overlap with writes,
// so the database grows more than needed while making actual reads interleaved
// with writes. So manage these completely seperately.

class LmDb {
  constructor(base, name, options) {
    this._env = void 0;
    this._dbi = void 0;
    this._path = void 0;
    this._rtxn = void 0;
    this._wtxn = void 0;
    this._env = new _nodeLmdb.default.Env();
    this._path = _path.default.join(base, name);
    this._rtxn = null;
    this._wtxn = null;

    _mkdirp.default.sync(this._path);

    const dbsize = this.size(true);
    const mapSize = Math.ceil(1 + dbsize / GB) * GB;
    l.debug(() => "Current mapsize set to ".concat((mapSize / GB).toFixed(1), "GB"));

    this._env.open({
      path: this._path,
      mapSize
    });
  }

  getNextMapSize() {
    const dbsize = this.size();

    const mapSize = this._env.info().mapSize;

    const remaining = (mapSize - dbsize) / GB;
    return remaining < 0.25 ? mapSize + GB : mapSize;
  }

  growMapSize() {
    const info = this._env.info();

    const next = this.getNextMapSize();

    if (next > info.mapSize) {
      l.debug(() => "Growing mapsize to ".concat((next / GB).toFixed(1), "GB"));

      this._env.resize(next);
    }
  }

  close() {
    this.exitRead();

    this._dbi.close();

    this._dbi = null;

    this._env.close();
  }

  open() {
    this._dbi = this._env.openDbi({
      create: true,
      keyIsBuffer: true
    });
    this.enterRead();
  }

  drop() {
    throw new Error('drop() not implemented');
  }

  empty() {
    throw new Error('empty() not implemented');
  }

  maintain(fn) {
    fn({
      isCompleted: true,
      keys: 0,
      percent: 100
    });
  }

  rename(base, file) {// nothing
  }

  size() {
    let withExistsSync = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;

    const db = _path.default.join(this._path, 'data.mdb');

    return !withExistsSync || _fs.default.existsSync(db) ? _fs.default.statSync(db).size : 0;
  }

  enterRead() {
    this._rtxn = this._env.beginTxn({
      readOnly: true
    });
  }

  exitRead() {
    this._rtxn.abort();

    this._rtxn = null;
  }

  txCommit() {
    this._wtxn.commit();

    this._wtxn = null;
    this.enterRead();
  }

  txRevert() {
    this._wtxn.abort();

    this._wtxn = null;
    this.enterRead();
  }

  txStart() {
    this.exitRead(); // grow between transactions

    this.growMapSize();
    this._wtxn = this._env.beginTxn();
  }

  del(key) {
    this._wtxn.del(this._dbi, (0, _util.u8aToBuffer)(key));
  }

  get(_key) {
    const value = this._rtxn.getBinary(this._dbi, (0, _util.u8aToBuffer)(_key));

    return value ? (0, _util.bufferToU8a)(value) : null;
  }

  put(key, value) {
    this._wtxn.putBinary(this._dbi, (0, _util.u8aToBuffer)(key), (0, _util.u8aToBuffer)(value));
  }

}

exports.default = LmDb;